@@ -30,7 +30,11 @@
 #include "gok-action.h"
 #include "gok-feedback.h"
 #include "gok-keyboard.h"
+#include "gok-gconf-keys.h" /* used only for uses_corepointer hack below */
 #include "main.h"
+#include "gok-repeat.h"
+
+#define REPEAT_OFF_NAME "repeat"
 
 /* pointer to the current access method */
 static GokAccessMethod* m_pAccessMethodCurrent;
@@ -115,7 +119,7 @@ static GokScannerEffect* m_pEffectsOnDwell;
 #define MAX_HANDLERS 30
 
 /* number of possible calls */
-#define MAX_CALL_NAMES 59
+#define MAX_CALL_NAMES 60
 
 /* names of all the possible effect calls */
 /* This must be in the same order as the enum "CallIds" */
@@ -137,7 +141,8 @@ static gchar ArrayCallNames[MAX_CALL_NAMES][24] ={
 "ChunkerIfLeft", "ChunkerIfRight",
 "ChunkerIfKeySelected", "HighlightCenterKey",
 "HighlightFirstChunk", "HighlightFirstKey", "SelectChunk",
-"ChunkerChunkHighlight", "UnhighlightAll",
+"ChunkerChunkHighlight", "UnhighlightAll", 
+"RepeatOn", /* "RepeatOff",  use StateReset */
 "Timer1Set", "Timer1Stop",
 "Timer2Set", "Timer2Stop",
 "Timer3Set", "Timer3Stop",
@@ -1618,6 +1623,8 @@ gboolean gok_scanner_change_method (gchar* NameAccessMethod)
 	}
 	
 	gok_log_x ("Warning: gok_scanner_change_method failed (%s)!", NameAccessMethod);
+	/* report bad access name to the user */
+	fprintf(stderr,"Unkown access method name: %s, aborting.",NameAccessMethod);
 	return FALSE;
 }
 
@@ -1649,7 +1656,7 @@ void gok_scanner_reset_access_method ()
 * @pState: Pointer to the new state.
 * @NameAccessMethod: Name of the access method that contains the state.
 *
-* Changes the state of the event handlers to the given state.
+* Maps event handlers to effects..  
 **/
 void gok_scanner_change_state (GokScannerState* pState, gchar* NameAccessMethod)
 {
@@ -1659,11 +1666,21 @@ void gok_scanner_change_state (GokScannerState* pState, gchar* NameAccessMethod)
 	gint stateEffect;
 	GokAction* pAction;
 
+	gok_log_enter();
+	gok_log("STATE: %s next state? %d", pState->NameState, pState->pStateNext);	
 	if (pState == NULL)
 	{
 		gok_log_x ("State is NULL!");
+		gok_log_leave();
 		return;
 	}
+	
+	/* stop all timers - TODO: test if this causes bad behaviour */
+	gok_scanner_timer_stop (1);
+	gok_scanner_timer_stop (2);
+	gok_scanner_timer_stop (3);
+	gok_scanner_timer_stop (4);
+	gok_scanner_timer_stop (5);
 
 	/* store a pointer to the current state */
 	m_pStateCurrent = pState;
@@ -1920,6 +1937,7 @@ void gok_scanner_change_state (GokScannerState* pState, gchar* NameAccessMethod)
 
 		pHandler = pHandler->pHandlerNext;
 	}
+	gok_log_leave();
 }
 
 /**
@@ -2385,7 +2403,9 @@ void gok_scanner_perform_effects (GokScannerEffect* pEffect)
 	gint param1Rate;
 	gint param2Rate;
 	gint counterNumber;
+	GokKey* pKey;
 
+	gok_log_enter();
 	/* we need the name of the access method to get the gok_data */
 	pAccessMethod = gok_scanner_get_current_access_method();
 	g_assert (pAccessMethod != NULL);
@@ -2393,6 +2413,7 @@ void gok_scanner_perform_effects (GokScannerEffect* pEffect)
 	/* perform the handler effect */
 	while (pEffect != NULL)
 	{
+		gok_log("looking at effect: [%s]",pEffect->pName);
 		EffectCodeReturned = 0;
 
 		switch (pEffect->CallId)
@@ -2619,6 +2640,10 @@ gok_log_x ("chunker  if key selected\n");
 			case CALL_CHUNKER_HIGHLIGHT_FIRST_KEY:
 				gok_chunker_highlight_first_key(); 
 				break;
+			
+			case CALL_SCANNER_REPEAT_ON:  
+				gok_scanner_repeat_on();
+				break;
 
 			case CALL_CHUNKER_SELECT_CHUNK:
 				EffectCodeReturned = gok_chunker_select_chunk();
@@ -2711,7 +2736,7 @@ gok_log_x ("chunker  if key selected\n");
 				{
 					gok_chunker_counter_set (counterNumber, param2Rate); 
 				}
-				else
+				else	
 				{
 					gok_log_x ("Can't get CHUNKER_COUNTER_SET param2 setting: %s!\n", pEffect->pName);
 				}
@@ -2742,7 +2767,20 @@ gok_log_x ("chunker  if key selected\n");
 				break;
 
 			case CALL_OUTPUT_SELECTEDKEY:
-				EffectCodeReturned = gok_keyboard_output_selectedkey (); 
+				pKey = gok_keyboard_output_selectedkey ();
+				if (gok_repeat_getArmed()){
+					if (gok_repeat_key(pKey)) {
+						gok_scanner_repeat_on();
+						/* to fix problem with scanning methods we
+						   must unfortunately cut the effect list short 
+						   here. TODO revisit when repeatoff state is added
+						   to xam files. */
+						gok_log("leaving effect abruptly");
+						gok_log_leave();
+						return;
+					}
+				}
+				EffectCodeReturned = 0;
 				break;
 
 			case CALL_SET_SELECTEDKEY:
@@ -2797,6 +2835,7 @@ gok_log_x ("chunker  if key selected\n");
 		/* is there a condition to be checked? */
 		if (pEffect->CompareType == COMPARE_EQUAL)
 		{
+			gok_log("compare ==");
 			/* yes, compare the value returned to the compare value */
 			if (EffectCodeReturned == pEffect->CompareValue)
 			{
@@ -2809,6 +2848,7 @@ gok_log_x ("chunker  if key selected\n");
 		}
 		else if (pEffect->CompareType == COMPARE_LESSTHAN)
 		{
+			gok_log("compare <");
 			if (EffectCodeReturned < pEffect->CompareValue)
 			{
 				gok_scanner_perform_effects (pEffect->pEffectTrue);
@@ -2820,6 +2860,7 @@ gok_log_x ("chunker  if key selected\n");
 		}
 		else if (pEffect->CompareType == COMPARE_GREATERTHAN)
 		{
+			gok_log("compare >");
 			if (EffectCodeReturned > pEffect->CompareValue)
 			{
 				gok_scanner_perform_effects (pEffect->pEffectTrue);
@@ -2831,6 +2872,7 @@ gok_log_x ("chunker  if key selected\n");
 		}
 		else if (pEffect->CompareType == COMPARE_EQUALORLESSTHAN)
 		{
+			gok_log("compare <=");
 			if (EffectCodeReturned <= pEffect->CompareValue)
 			{
 				gok_scanner_perform_effects (pEffect->pEffectTrue);
@@ -2842,6 +2884,7 @@ gok_log_x ("chunker  if key selected\n");
 		}
 		else if (pEffect->CompareType == COMPARE_EQUALORGREATERTHAN)
 		{
+			gok_log("compare >=");
 			if (EffectCodeReturned >= pEffect->CompareValue)
 			{
 				gok_scanner_perform_effects (pEffect->pEffectTrue);
@@ -2855,6 +2898,8 @@ gok_log_x ("chunker  if key selected\n");
 		/* move on to the next effect in the handler */
 		pEffect = pEffect->pEffectNext;
 	}
+	gok_log("leaving effect.");
+	gok_log_leave();
 }
 
 /**
@@ -3122,6 +3167,49 @@ GokScannerState* gok_scanner_get_current_state ()
 	return m_pStateCurrent;
 }
 
+
+/**
+ * gok_scanner_current_state_uses_corepointer:
+ *
+ * Returns: TRUE if the current scanner state has a handler that is
+ * currently configured to use an action which is connected to the corepointer
+ * (button or dwell), and FALSE otherwise.
+ */
+gboolean 
+gok_scanner_current_state_uses_corepointer (void)
+{
+	gboolean retval = FALSE;
+	GokAccessMethod *method;
+
+	if  (m_pEffectsLeftButtonDown ||
+	     m_pEffectsLeftButtonUp ||
+	     m_pEffectsRightButtonDown ||
+	     m_pEffectsRightButtonUp ||
+	     m_pEffectsMiddleButtonDown ||
+	     m_pEffectsMiddleButtonUp ||
+	     m_pEffectsMouseButton4Down ||
+	     m_pEffectsMouseButton4Up ||
+	     m_pEffectsMouseButton5Down ||
+	     m_pEffectsMouseMovement) {
+		retval = TRUE;
+	}
+	/* FIXME! hack, assume we don't use corepointer if we have a valid input device */
+	/* This is also inefficient since we don't cache the result */
+	else {
+		char *input_device_name = NULL;
+		gok_gconf_get_string (gconf_client_get_default (), 
+				      GOK_GCONF_INPUT_DEVICE, 
+				      &input_device_name);
+		if (input_device_name == NULL)
+			retval = TRUE;
+		else
+			g_free (input_device_name);
+	}
+
+	return retval;
+}
+
+
 /**
  * gok_scanner_current_state_uses_core_mouse_button:
  * @button:
@@ -3260,3 +3348,34 @@ gboolean gok_scanner_get_multiple_rates (gchar* NameAccessMethod, gchar* NameSet
 	
 	return TRUE;
 }
+
+
+/**
+* gok_scanner_repeat_on
+*
+* Changes the state of the current access method so that the user can
+* stop the repeating with an action.
+*
+* returns: void.
+**/
+void 
+gok_scanner_repeat_on(void)
+{
+	GokAccessMethod* pAccessMethod;
+	GokScannerState* pState;
+	
+	gok_log_enter();
+	pAccessMethod = gok_scanner_get_current_access_method();
+	g_assert (pAccessMethod != NULL);
+	pState = pAccessMethod->pStateFirst;
+	while (pState != NULL)
+	{
+		if ( strcmp(pState->NameState, REPEAT_OFF_NAME) == 0 )
+		{
+			gok_scanner_change_state (pState, pAccessMethod->Name);
+			break; /* from while */
+		}
+		pState = pState->pStateNext;
+	}
+	gok_log_leave();
+}
