@@ -216,9 +216,19 @@ static struct poptOption options[] = {
 };
 
 /* private */
-static int gok_segv_handler(int sig)
+static int gok_sig_handler(int sig)
 {
-	fprintf (stderr, "gok: Segmentation fault\n");
+	switch (sig) {
+	case SIGSEGV:
+		fprintf (stderr, "gok: Segmentation fault\n");
+		break;
+	case SIGTERM:
+		fprintf (stderr, "gok: exiting (terminated)\n");
+		break;
+	case SIGINT:
+	default:
+		break;
+	}
 
 	gok_main_close();
 
@@ -428,8 +438,10 @@ gint gok_main_open(gint argc, gchar *argv[])
 		return GOK_ERROR_DISPLAY_SCAN_MAIN;
 	}
 
-	/* connect a segv handler */
-	signal(SIGSEGV, (void (*)(int))gok_segv_handler);
+	/* connect signal handlers; don't trap SIGKILL */ 
+	signal(SIGSEGV, (void (*)(int))gok_sig_handler);
+	signal(SIGTERM, (void (*)(int))gok_sig_handler);
+	signal(SIGINT,  (void (*)(int))gok_sig_handler);
 
 	return 0;
 }
@@ -953,6 +965,36 @@ gboolean gok_main_display_scan_previous ()
 	return bReturnCode;
 }
 
+GokKeyboard *
+gok_main_keyboard_find_byname (const gchar *NameKeyboard)
+{
+	GokKeyboard *pKB = NULL;
+	
+	if (NameKeyboard == NULL)
+	{
+		/* if keyboard name is NULL then use first keyboard in list */
+		pKB = m_pKeyboardFirst;
+	}
+	else
+	{
+		/* find the keyboard in the list (according to its name) */
+		pKB = m_pKeyboardFirst;
+		while (pKB != NULL)
+		{
+			if (g_strcasecmp (pKB->Name, NameKeyboard) == 0)
+			{
+				/* found the keyboard */
+				break;
+			}
+			else
+			{
+				pKB = pKB->pKeyboardNext;
+			}
+		}
+	}
+	return pKB;
+}
+
 /**
 * gok_main_display_scan
 * @pKeyboard: If this is supplied it takes precedence over the name parameter (useful for dynamic keyboards with the same name)
@@ -986,28 +1028,7 @@ gboolean gok_main_display_scan (GokKeyboard* pKeyboard, gchar* NameKeyboard, Key
 	}
 	else
 	{
-		/* if keyboard name is NULL then use first keyboard in list */
-		if (NameKeyboard == NULL)
-		{
-			pKB = m_pKeyboardFirst;
-		}
-		else
-		{
-			/* find the keyboard in the list (according to its name) */
-			pKB = m_pKeyboardFirst;
-			while (pKB != NULL)
-			{
-				if (g_strcasecmp (pKB->Name, NameKeyboard) == 0)
-				{
-					/* found the keyboard */
-					break;
-				}
-				else
-				{
-					pKB = pKB->pKeyboardNext;
-				}
-			}
-		}
+		pKB = gok_main_keyboard_find_byname (NameKeyboard);
 	}
 
 	if (pKB == NULL)
